==> Mtftp-0.6.tar.gz: Mtftp-0.6/source/Tftpd.java <==
/*
    Tftpd.java - a TFTP Protocol Version 2 Server
    Copyright (C) 1999 Mark Benvenuto <ivymccough@worldnet.att.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package gnu.inet.tftp;

//import java.lang.*;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InterruptedIOException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.TimeZone;
/**
Mark's Tftp Server<BR>
Trival File Transfer Protocol (RFC 1350)<BR>
<BR>
Mark Benvenuto <ivymccough@worldnet.att.net><BR>
Java JDK 1.1.5<BR>
<BR>
A TFTP server conforming to TFTP prtocol version 2 (RFC 1360)<BR>
Under GNU License Version 2.<BR>
<BR>
Version History<BR>
<BR>
	Check NEWS & ChangeLog files<BR>

@author Mark Benvenuto
@version 0.6
@see TftpServerConnection

*/

public class Tftpd
{
	//Properties
	String rootDirectory = "tftpdroot";
	int logging = 1;
	int logStdout = 1;
	int logUsingSyslogd = 0; //to be implmented in next version
	int socketTimeout = 3000;
	//int maxRetries = 5;
	int debugMode = 0;
	String logFile = "tftpd.log";
	//Logging Stuff
	private File lFile = null;
	private FileOutputStream fout = null;
	private TftpLogStream logTftpStream = null;
	private PrintStream logStream = null;
	//Date Stuff
	//private Date logDate = null;
	//private SimpleDateFormat logDateFormat = null;
	//Network Socket Stuff
	InetAddress iahost = null;

	/**
		Constructs a new Tftpd server
	*/
	public Tftpd()
	{
		if( loadProperties("tftpd.properties") == -1)
		{
			System.exit(-1);
		}

		initializeLogging();

		logStream.println("Java tftpd Version 0.6 by Mark Benvenuto <ivymmcough@worldnet.att.net>");
		
		try {
			iahost.getLocalHost();
			logStream.println("Host: " + iahost.getHostName() + "(" + iahost.getAddress() + ")");
		} catch( UnknownHostException uhe)
		{
			logStream.println("Unknown local host. Must not be connected " +
					   "to network. Address is localhost(127.0.0.1)");
			logStream.println("Host: localhost(127.0.0.1)");
		}

		logStream.println("Properties Dump...");
		logStream.println("rootDirectory: " + rootDirectory );
		logStream.println("logging: " + logging );
		logStream.println("logFile: " + logFile );
		logStream.println("logStdout: " + logStdout );
		logStream.println("socketTimeout: " + socketTimeout );
		//logStream.println("maxRetries: " + maxRetries );
		logStream.println("debugMode: " + debugMode );
		
	}

	/**
		Initializes Logging Based on global variables
	*/
	private void initializeLogging()
	{
		if( logging == 1) {
			System.out.println("Initializing Logging...");

			//First try syslogd if user wants, 
			//then file user chose, then stdout
			if( logUsingSyslogd == 1) {
				//TODO: do something here in the next version
			}

			try {
				if( logStdout == 0)
				{
					lFile = new File(logFile);
					fout = new FileOutputStream( lFile );
					logTftpStream = new TftpLogStream();
					logTftpStream.setLogPrimary( new PrintStream( new BufferedOutputStream(fout) ) );
					logTftpStream.useLogPrimary(true);
					logStream = new PrintStream( new LineBufferedOutputStream( logTftpStream ) );
				} else if ( logStdout == 1)
				{
					logStream = System.out;
				}
			} catch ( IOException ioe) {
				System.out.println("Log file not found.");
				System.out.println("Exception: " + ioe.getMessage());
				ioe.printStackTrace(System.out);
				logStream = System.out;
			}
		}
	}

	/**
		Gets the PrintStream attached to the current log stream
	*/
	public PrintStream getLogWriter()
	{
		return logStream;
	}

	/*
		Logs the message to file stream or System.out

		@param str Message to log
	*/
	/*NOT NEEDED BECAUSE OF TftpLogStream
	public void log(String str)
	{
		logDate.setTime(System.currentTimeMillis());
		logStream.println( logDateFormat.format(logDate) + " : " + str);
		//System.out.println( logDateFormat.format(logDate) + " : " + str);
	}*/

	/**
		Load properties file and initialize global variables

		@param filename File that contains properties
		@return -1 indicates failed to load, otherwise 0
	*/
	private int loadProperties( String filename)
	{
		File pfile = new File( filename );
		if( pfile.exists() == false) {
			System.out.println("Properties file not found. Exiting...");
			return -1;
		}
		try {
			FileInputStream fin = new FileInputStream( pfile );
			Properties props = new Properties();
			props.load(fin);

			//Load Properties		
			rootDirectory = props.getProperty( "rootDirectory", "tftpdroot" );
			logging = Integer.parseInt( props.getProperty("logging", "1" ), 10);
			logFile = props.getProperty("logFile", "tftpd.log");
			logUsingSyslogd = Integer.parseInt( props.getProperty("logUsingSyslogd", "1" ), 10);
			logStdout = Integer.parseInt( props.getProperty("logStdout", "1" ), 10);
			socketTimeout = Integer.parseInt( props.getProperty( "socketTimeout", "3000"), 10);
			if( socketTimeout < 0)
				socketTimeout = 3000; //Cannot have negative times now can we
			//maxRetries = Integer.parseInt( props.getProperty("maxRetries", "5"), 10);
			debugMode = Integer.parseInt( props.getProperty("debugMode", "0	"), 10);
			return 0;
		} catch ( FileNotFoundException fnfe) {
			System.out.println("Properties file not found. Exiting...");
			System.out.println("Exception: " + fnfe.getMessage());
			fnfe.printStackTrace(System.out);
			return -1;
		} catch ( IOException ioe) {
			System.out.println("Properties file failed to load. Exiting...");
			System.out.println("Exception: " + ioe.getMessage());
			ioe.printStackTrace(System.out);
			return -1;
		}
	}

	/**
		Main Server Loop. Processes all incoming RRQ/WRQ requests 
		and spawns necessary TftpServerConnections

		@see TftpServerConnection
	*/
	public void processConnections()
	{
		if( debugMode > 0 )
			logStream.println("Staring Main Server Loop...");
		int localport = 69; //Default TFTP port
		
		int maxPacketSize = 516;
		int opcode = 0;
		byte data[] = new byte[maxPacketSize];
		DatagramPacket pdata =new DatagramPacket(data, maxPacketSize);

		String filename = null;
		InetAddress ria=null;
		int remoteport=0;

		TftpServerConnection tftpsc = null;

		try {
			DatagramSocket mainSock = new DatagramSocket( localport );
			if( debugMode > 0 )
				logStream.println("Created Main Socket: " + mainSock);

			mainSock.setSoTimeout( socketTimeout );
			for(;;)
				try {
					mainSock.receive( pdata );

					//See what type of Packet it is
					opcode = data[1];
					if( debugMode > 0 )
						logStream.println("Received Packet of Type: " + opcode);

					ria = pdata.getAddress();
					remoteport = pdata.getPort();
					filename = readString(data, 2);

					//Make a formal log of from who and what they want
					if( opcode == 1) //We have a RRQ
						logStream.println("Received Read Request from " + ria + " for " + filename);
					else if( opcode == 2) //We have a WRQ
						logStream.println("Received Write Request from " + ria + " for " + filename);


					if( debugMode > 0 )
						logStream.println("Received Packet from: " + ria + ":" + remoteport);

					filename = rootDirectory + File.separator + filename;
					
					if( opcode == 1) { //We have a RRQ
						//Start up a new ServerConnection, and hand off processing
						tftpsc = new TftpServerConnection(ria, remoteport, filename, 1, logStream, debugMode);
						tftpsc.setSocketTimeout( socketTimeout );
						Thread th = new Thread( tftpsc );
						tftpsc = null;
						th.start();
					}
					else if( opcode == 2) { //We have a WRQ
						//Start up a new ServerConnection, and hand off processing
						tftpsc = new TftpServerConnection(ria, remoteport, filename, 1, logStream, debugMode);
						tftpsc.setSocketTimeout( socketTimeout );
						Thread th = new Thread( tftpsc );
						tftpsc = null;
						th.start();
					}
					else {
						logStream.println("Unknown opcode: " + opcode + " from " + pdata.getAddress() );
					}

				} catch( InterruptedIOException iioe) {
					if( debugMode > 1 )
						logStream.println("Socket Timedout. Trying again.");
					logStream.flush();
				}
				catch( IOException ioe) {
					logStream.println("Error: Main Socket Failed with IO Exception");
					//System.out.println("Error: Main Socket Failed with IO Exception");
					ioe.printStackTrace( logStream );
					//ioe.printStackTrace( System.out );
				}
		} catch (SocketException se) {
			logStream.println("Error: Failed to open Main Server Socket");
			se.printStackTrace( logStream );
		}
	}
	/*
		Used to read a string from an array of bytes, like a packet
	
		@param rs Bytes from a packet to read for string
		@param off Offset is the position to start reading from
		@return A String from the packet, could be zero length
	*/
	protected String readString(byte[] rs, int off)
	{
		String s = new String();
		off--;
		while(rs[++off]!=(byte)'\0')
			s += String.valueOf((char)rs[off]);
		return s;
	}

	/**
		Creates a Tftpd server and start processing connections
	*/
	public static void main(String args[])
	{
		Tftpd server;
		server = new Tftpd();
		server.processConnections();
	}
}

==> Mtftp-0.7.tar.gz: Mtftp-0.7/source/Tftpd.java <==
/*
    Tftpd.java - a TFTP Protocol Version 2 Server
    Copyright (C) 1999 Mark Benvenuto <mcb54@columbia.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package gnu.inet.tftp;

//import java.lang.*;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InterruptedIOException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.TimeZone;
/**
Mark's Tftp Server<BR>
Trival File Transfer Protocol (RFC 1350)<BR>
<BR>
Mark Benvenuto <mcb54@columbia.edu><BR>
Java JDK 1.1.8<BR>
<BR>
A TFTP server conforming to TFTP prtocol version 2 (RFC 1360)<BR>
Under GNU License Version 2.<BR>
<BR>
Version History<BR>
<BR>
	Check NEWS & ChangeLog files<BR>

@author Mark Benvenuto
@version 0.7
@see TftpServerConnection

*/

public class Tftpd
{
	//Properties
	String rootDirectory = "tftpdroot";
	int logging = 1;
	int logStdout = 1;
	int logUsingSyslogd = 0; //to be implmented in next version
	int socketTimeout = 3000;
	//int maxRetries = 5;
	int debugMode = 0;
	String logFile = "tftpd.log";
	//Logging Stuff
	private File lFile = null;
	private FileOutputStream fout = null;
	private TftpLogStream logTftpStream = null;
	private PrintStream logStream = null;
	//Date Stuff
	//private Date logDate = null;
	//private SimpleDateFormat logDateFormat = null;
	//Network Socket Stuff
	InetAddress iahost = null;

	/**
		Constructs a new Tftpd server
	*/
	public Tftpd()
	{
		if( loadProperties("tftpd.properties") == -1)
		{
			System.exit(-1);
		}

		initializeLogging();

		logStream.println("Java tftpd Version 0.7 by Mark Benvenuto <mcb54@columbia.edu>");
		
		try {
			iahost = iahost.getLocalHost();
			logStream.println("Host: " + iahost.getHostName() + "(" + iahost.getAddress() + ")");
		} catch( UnknownHostException uhe)
		{
			logStream.println("Unknown local host. Must not be connected " +
					   "to network. Address is localhost(127.0.0.1)");
			logStream.println("Host: localhost(127.0.0.1)");
		}

		logStream.println("Properties Dump...");
		logStream.println("rootDirectory: " + rootDirectory );
		logStream.println("logging: " + logging );
		logStream.println("logFile: " + logFile );
		logStream.println("logStdout: " + logStdout );
		logStream.println("socketTimeout: " + socketTimeout );
		//logStream.println("maxRetries: " + maxRetries );
		logStream.println("debugMode: " + debugMode );
		
	}

	/**
		Initializes Logging Based on global variables
	*/
	private void initializeLogging()
	{
		if( logging == 1) {
			System.out.println("Initializing Logging...");

			//First try syslogd if user wants, 
			//then file user chose, then stdout
			if( logUsingSyslogd == 1) {
				//TODO: do something here in the next version
			}

			try {
				if( logStdout == 0)
				{
					lFile = new File(logFile);
					fout = new FileOutputStream( lFile );
					logTftpStream = new TftpLogStream();
					logTftpStream.setLogPrimary( new PrintStream( new BufferedOutputStream(fout) ) );
					logTftpStream.useLogPrimary(true);
					logStream = new PrintStream( new LineBufferedOutputStream( logTftpStream ) );
				} else if ( logStdout == 1)
				{
					logStream = System.out;
				}
			} catch ( IOException ioe) {
				System.out.println("Log file not found.");
				System.out.println("Exception: " + ioe.getMessage());
				ioe.printStackTrace(System.out);
				logStream = System.out;
			}
		}
	}

	/**
		Gets the PrintStream attached to the current log stream
	*/
	public PrintStream getLogWriter()
	{
		return logStream;
	}

	/*
		Logs the message to file stream or System.out

		@param str Message to log
	*/
	/*NOT NEEDED BECAUSE OF TftpLogStream
	public void log(String str)
	{
		logDate.setTime(System.currentTimeMillis());
		logStream.println( logDateFormat.format(logDate) + " : " + str);
		//System.out.println( logDateFormat.format(logDate) + " : " + str);
	}*/

	/**
		Load properties file and initialize global variables

		@param filename File that contains properties
		@return -1 indicates failed to load, otherwise 0
	*/
	private int loadProperties( String filename)
	{
		File pfile = new File( filename );
		if( pfile.exists() == false) {
			System.out.println("Properties file not found. Exiting...");
			return -1;
		}
		try {
			FileInputStream fin = new FileInputStream( pfile );
			Properties props = new Properties();
			props.load(fin);

			//Load Properties		
			rootDirectory = props.getProperty( "rootDirectory", "tftpdroot" );
			logging = Integer.parseInt( props.getProperty("logging", "1" ), 10);
			logFile = props.getProperty("logFile", "tftpd.log");
			logUsingSyslogd = Integer.parseInt( props.getProperty("logUsingSyslogd", "1" ), 10);
			logStdout = Integer.parseInt( props.getProperty("logStdout", "1" ), 10);
			socketTimeout = Integer.parseInt( props.getProperty( "socketTimeout", "3000"), 10);
			if( socketTimeout < 0)
				socketTimeout = 3000; //Cannot have negative times now can we
			//maxRetries = Integer.parseInt( props.getProperty("maxRetries", "5"), 10);
			debugMode = Integer.parseInt( props.getProperty("debugMode", "0	"), 10);
			return 0;
		} catch ( FileNotFoundException fnfe) {
			System.out.println("Properties file not found. Exiting...");
			System.out.println("Exception: " + fnfe.getMessage());
			fnfe.printStackTrace(System.out);
			return -1;
		} catch ( IOException ioe) {
			System.out.println("Properties file failed to load. Exiting...");
			System.out.println("Exception: " + ioe.getMessage());
			ioe.printStackTrace(System.out);
			return -1;
		}
	}

	/**
		Main Server Loop. Processes all incoming RRQ/WRQ requests 
		and spawns necessary TftpServerConnections

		@see TftpServerConnection
	*/
	public void processConnections()
	{
		if( debugMode > 0 )
			logStream.println("Staring Main Server Loop...");
		int localport = 69; //Default TFTP port
		
		int maxPacketSize = 516;
		int opcode = 0;
		byte data[] = new byte[maxPacketSize];
		DatagramPacket pdata =new DatagramPacket(data, maxPacketSize);

		String filename = null;
		InetAddress ria=null;
		int remoteport=0;

		TftpServerConnection tftpsc = null;

		try {
			DatagramSocket mainSock = new DatagramSocket( localport );
			if( debugMode > 0 )
				logStream.println("Created Main Socket: " + mainSock);

			mainSock.setSoTimeout( socketTimeout );
			for(;;)
				try {
					mainSock.receive( pdata );

					//See what type of Packet it is
					opcode = data[1];
					if( debugMode > 0 )
						logStream.println("Received Packet of Type: " + opcode);

					ria = pdata.getAddress();
					remoteport = pdata.getPort();
					filename = readString(data, 2);

					//Make a formal log of from who and what they want
					if( opcode == 1) //We have a RRQ
						logStream.println("Received Read Request from " + ria + " for " + filename);
					else if( opcode == 2) //We have a WRQ
						logStream.println("Received Write Request from " + ria + " for " + filename);


					if( debugMode > 0 )
						logStream.println("Received Packet from: " + ria + ":" + remoteport);

					filename = rootDirectory + File.separator + filename;
					
					if( opcode == 1) { //We have a RRQ
						//Start up a new ServerConnection, and hand off processing
						tftpsc = new TftpServerConnection(ria, remoteport, filename, 1, logStream, debugMode);
						tftpsc.setSocketTimeout( socketTimeout );
						Thread th = new Thread( tftpsc );
						tftpsc = null;
						th.start();
					}
					else if( opcode == 2) { //We have a WRQ
						//Start up a new ServerConnection, and hand off processing
						tftpsc = new TftpServerConnection(ria, remoteport, filename, 2, logStream, debugMode);
						tftpsc.setSocketTimeout( socketTimeout );
						Thread th = new Thread( tftpsc );
						tftpsc = null;
						th.start();
					}
					else {
						logStream.println("Unknown opcode: " + opcode + " from " + pdata.getAddress() );
					}

				} catch( InterruptedIOException iioe) {
					if( debugMode > 1 )
						logStream.println("Socket Timedout. Trying again.");
					logStream.flush();
				}
				catch( IOException ioe) {
					logStream.println("Error: Main Socket Failed with IO Exception");
					//System.out.println("Error: Main Socket Failed with IO Exception");
					ioe.printStackTrace( logStream );
					//ioe.printStackTrace( System.out );
				}
		} catch (SocketException se) {
			logStream.println("Error: Failed to open Main Server Socket");
			se.printStackTrace( logStream );
		}
	}
	/*
		Used to read a string from an array of bytes, like a packet
	
		@param rs Bytes from a packet to read for string
		@param off Offset is the position to start reading from
		@return A String from the packet, could be zero length
	*/
	protected String readString(byte[] rs, int off)
	{
		String s = new String();
		off--;
		while(rs[++off]!=(byte)'\0')
			s += String.valueOf((char)rs[off]);
		return s;
	}

	/**
		Creates a Tftpd server and start processing connections
	*/
	public static void main(String args[])
	{
		Tftpd server;
		server = new Tftpd();
		server.processConnections();
	}
}
